#========================================================================================
# package      DE_EPAGES::Ipayment::t::integration::ChangeActionTrigger_CancelCallback
# base         DE_EPAGES::Shop::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration test
#========================================================================================
package DE_EPAGES::Ipayment::t::integration::ChangeActionTrigger_CancelCallback;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use base DE_EPAGES::Shop::API::TestImportXML;

use Test::More;

use strict;

#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in other to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'TestShopChangeActionIpayment';
}
sub importXMLPath {
    return 'ShopXMLs/ChangeActionTrigger_CancelCallback.xml';
}

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;
    $self->testCancelCallbackRequest();
    $self->testCaptureCallbackRequest();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub testCancelCallbackRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderPath = $self->shopPath.'/Customers/TestCustomer/Orders/1234-5678';
    my $CustomerOrder = LoadObjectByPath($OrderPath);
    my $DataBaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    ok ($DataBaseBasketOrOrder->extractFromBasketOrOrder($CustomerOrder),"$SubTestName - tests if extractFromBasketOrOrder works fine");
    my $Shop = $CustomerOrder->getSite();
    my $CancelCallbackURL = BuildShopUrl(
        $Shop,
        {'ChangeAction' => 'CancelCallback'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );
     $DataBaseBasketOrOrder->getHiddenTriggerUrl();
    note('Used CancelCallbackURL: '.$CancelCallbackURL);
    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post($CancelCallbackURL, {'ThisIs' => 'anIntegrationTest for CancelCallback'});

    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200 ok");
}

#----------------------------------------------------------------------------------------
sub testCaptureCallbackRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderPath = $self->shopPath.'/Customers/TestCustomer/Orders/1234-5678';
    my $CustomerOrder = LoadObjectByPath($OrderPath);
    my $DataBaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    ok ($DataBaseBasketOrOrder->extractFromBasketOrOrder($CustomerOrder),"$SubTestName - tests if extractFromBasketOrOrder works fine");
    my $Shop = $CustomerOrder->getSite();
    my $CaptureCallbackURL = BuildShopUrl(
        $Shop,
        {'ChangeAction' => 'CaptureCallback'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );
    $DataBaseBasketOrOrder->getHiddenTriggerUrl();
    note('Used CancelCallbackURL: '.$CaptureCallbackURL);
    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post($CaptureCallbackURL, {'ThisIs' => 'anIntegrationTest for CaptureCallback'});

    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200 ok");
}
__PACKAGE__->runScript();

1;