#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CGI_AuthOrPreauth
# state        public
#----------------------------------------------------------------------------------------
# description  Unit tests for API::CGI::AuthOrPreauth
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CGI_AuthOrPreauth;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;
use DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::More;

use strict;



sub Main {

    Test_AuthOrPreauth_Normal();
    Test_AuthOrPreauth_Paypal();
    Test_AuthOrPreauth_BillSAFE();

    done_testing();

}



sub Test_AuthOrPreauth_Normal {
    my $SubTestName = (caller(0))[3];


    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    $AuthOrPreauth = Test::MockObject::Extends->new($AuthOrPreauth);

    $AuthOrPreauth->mock('_getDatabaseBasketOrOrder', sub {
        return _createDatabaseBasketOrOrder('TestPaymentType');
    });

    my $Order = _createOrder();
    my $hFormData = $AuthOrPreauth->getRequestParameters($Order);
    my $HiddenInputs = $hFormData->{'HiddenInputs'};

    # check standard parameters
    is($HiddenInputs->{'redirect_action'}, 'post', "$SubTestName - test if redirect action is correct");
    is($HiddenInputs->{'ipayment_session_id'}, 'TestSessionID', "$SubTestName - test if session id is correct");
    is($HiddenInputs->{'shopper_id'}, 'TestGUID', "$SubTestName - test if shopper_id is correct");
    is($HiddenInputs->{'advanced_strict_id_check'}, 'true', "$SubTestName - test if advanced_strict_id_check is set");
    is($HiddenInputs->{'trx_typ'}, 'TestTransactionType', "$SubTestName - test if transaction type is correct");
    is($HiddenInputs->{'trx_paymenttyp'}, 'TestPaymentType', "$SubTestName - test if payment type is correct");
    is($HiddenInputs->{'addr_name'}, 'TestFullName', "$SubTestName - test if name is correct");
    is($HiddenInputs->{'addr_street'}, 'TestStreet', "$SubTestName - test if street is correct");
    is($HiddenInputs->{'addr_city'}, 'TestCity', "$SubTestName - test if city is correct");
    is($HiddenInputs->{'addr_zip'}, 'TestZipcode', "$SubTestName - test if zip code is correct");
    is($HiddenInputs->{'addr_country'}, 'TESTCODE2', "$SubTestName - test if country is correct");
    is($HiddenInputs->{'addr_email'}, 'TestEMail', "$SubTestName - test if email is correct");

}



sub Test_AuthOrPreauth_Paypal {
    my $SubTestName = (caller(0))[3];


    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    $AuthOrPreauth = Test::MockObject::Extends->new($AuthOrPreauth);

    $AuthOrPreauth->mock('_getDatabaseBasketOrOrder', sub {
        return _createDatabaseBasketOrOrder('paypal');
    });

    my $Order = _createOrder();
    my $hFormData = $AuthOrPreauth->getRequestParameters($Order);
    my $HiddenInputs = $hFormData->{'HiddenInputs'};

    # check standard parameters
    is($HiddenInputs->{'redirect_action'}, 'post', "$SubTestName - test if redirect action is correct");
    is($HiddenInputs->{'ipayment_session_id'}, 'TestSessionID', "$SubTestName - test if session id is correct");
    is($HiddenInputs->{'shopper_id'}, 'TestGUID', "$SubTestName - test if shopper_id is correct");
    is($HiddenInputs->{'advanced_strict_id_check'}, 'true', "$SubTestName - test if advanced_strict_id_check is set");
    is($HiddenInputs->{'trx_typ'}, 'TestTransactionType', "$SubTestName - test if transaction type is correct");
    is($HiddenInputs->{'trx_paymenttyp'}, 'paypal', "$SubTestName - test if payment type is correct");
    is($HiddenInputs->{'addr_name'}, 'TestFullName', "$SubTestName - test if name is correct");
    is($HiddenInputs->{'addr_street'}, 'TestStreet', "$SubTestName - test if street is correct");
    is($HiddenInputs->{'addr_city'}, 'TestCity', "$SubTestName - test if city is correct");
    is($HiddenInputs->{'addr_zip'}, 'TestZipcode', "$SubTestName - test if zip code is correct");
    is($HiddenInputs->{'addr_country'}, 'TESTCODE2', "$SubTestName - test if country is correct");
    is($HiddenInputs->{'addr_email'}, 'TestEMail', "$SubTestName - test if email is correct");

    # check additional paypal parameters
    is($HiddenInputs->{'silent'}, 'true', "$SubTestName - test if silent mode is activated");
    is($HiddenInputs->{'paypal_flow'}, 'ECM', "$SubTestName - test if paypal_flow is ECM");
}



sub Test_AuthOrPreauth_BillSAFE {
    my $SubTestName = (caller(0))[3];

    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    $AuthOrPreauth = Test::MockObject::Extends->new($AuthOrPreauth);

    $AuthOrPreauth->mock('_getDatabaseBasketOrOrder', sub {
        my $DatabaseBasketOrOrder = _createDatabaseBasketOrOrder('invoice');
        $DatabaseBasketOrOrder = _addParamsForBillsafeToMockedDatabaseBasketOrOrder($DatabaseBasketOrOrder);
        return $DatabaseBasketOrOrder;
    });
    $AuthOrPreauth->mock('_getBillsafeArticleList', sub {
        return _createBillsafeArticleList();
    });

    my $Order = _createOrder();
    my $hFormData = $AuthOrPreauth->getRequestParameters($Order);
    my $HiddenInputs = $hFormData->{'HiddenInputs'};

    # check standard parameters
    is($HiddenInputs->{'redirect_action'}, 'post', "$SubTestName - test if redirect action is correct");
    is($HiddenInputs->{'ipayment_session_id'}, undef, "$SubTestName - test if session id is not set");
    is($HiddenInputs->{'shopper_id'}, 'TestGUID', "$SubTestName - test if shopper_id is correct");
    is($HiddenInputs->{'advanced_strict_id_check'}, 'true', "$SubTestName - test if advanced_strict_id_check is set");
    is($HiddenInputs->{'trx_typ'}, 'TestTransactionType', "$SubTestName - test if transaction type is correct");
    is($HiddenInputs->{'trx_paymenttyp'}, 'invoice', "$SubTestName - test if payment type is correct");
    is($HiddenInputs->{'addr_name'}, 'TestFullName', "$SubTestName - test if name is correct");
    is($HiddenInputs->{'addr_street'}, 'TestStreet', "$SubTestName - test if street is correct");
    is($HiddenInputs->{'addr_city'}, 'TestCity', "$SubTestName - test if city is correct");
    is($HiddenInputs->{'addr_zip'}, 'TestZipcode', "$SubTestName - test if zip code is correct");
    is($HiddenInputs->{'addr_country'}, 'TESTCODE2', "$SubTestName - test if country is correct");
    is($HiddenInputs->{'addr_email'}, 'TestEMail', "$SubTestName - test if email is correct");
    is($HiddenInputs->{'client_name'}, 'ePages', "$SubTestName - test if client name is correct");
    is($HiddenInputs->{'client_version'}, '6.6.6', "$SubTestName - test if client version is correct");
    is($HiddenInputs->{'commission_partner_code'}, 'testPartnerId', "$SubTestName - test if partner id is correct");


    # check additional BillSAFE parameters
    is($HiddenInputs->{'silent'}, 'true', "$SubTestName - test if silent mode is activated");
    is($HiddenInputs->{'trxuser_id'}, 'TestUserID', "$SubTestName - test if transaction id is correct");
    is($HiddenInputs->{'trxpassword'}, 'TestUserPassword', "$SubTestName - test if transcation password is correct");
    is($HiddenInputs->{'trx_amount'}, '12345', "$SubTestName - test if amount is correct");
    is($HiddenInputs->{'trx_currency'}, 'TESTCURR', "$SubTestName - test if currency is correct");
    is($HiddenInputs->{'redirect_url'}, 'redirect.url', "$SubTestName - test if RedirectUrl is correct");
    is($HiddenInputs->{'hidden_trigger_url'}, 'hiddentrigger.url', "$SubTestName - test if HiddenTriggerUrl is correct");
    is($HiddenInputs->{'invoice_text'}, 'TestArticleList', "$SubTestName - test if article list is correct");
}



# Order can be empty, because it is only used over DB::BasketOrOrder or BillsafeArticleList which are mocked
sub _createOrder {
    return {};
}



sub _createDatabaseBasketOrOrder($) {
    my ($PaymentType) = @_;

    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    my $MockedDatabaseBasketOrOrder = Test::MockObject::Extends->new($DatabaseBasketOrOrder);

    $MockedDatabaseBasketOrOrder->mock('extractFromBasketOrOrder', sub{
        return undef;
    });
    $MockedDatabaseBasketOrOrder->mock('getAccountID', sub{
        return 'TestAccountID';
    });
    $MockedDatabaseBasketOrOrder->mock('getGUID', sub{
        return 'TestGUID';
    });
    $MockedDatabaseBasketOrOrder->mock('getBillingAddress', sub{
        return _createBillingAddress();
    });
    $MockedDatabaseBasketOrOrder->mock('getPaymentType', sub{
        return $PaymentType;
    });
    $MockedDatabaseBasketOrOrder->mock('getTransactionType', sub{
        return 'TestTransactionType';
    });
    $MockedDatabaseBasketOrOrder->mock('getSessionID', sub{
        return 'TestSessionID';
    });
    $MockedDatabaseBasketOrOrder->mock('_LoadRootObject', sub {
        my $hAttributes = {
            'Ipayment_PartnerId' => 'testPartnerId',
            'EpagesVersion' => '6.6.6',
        };
        my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('System', $hAttributes);
        return $MockedSystem;
    });

    return $MockedDatabaseBasketOrOrder;
}



sub _addParamsForBillsafeToMockedDatabaseBasketOrOrder($) {
    my ($MockedDatabaseBasketOrOrder) = @_;

    $MockedDatabaseBasketOrOrder->mock('getUserID', sub{
        return 'TestUserID';
    });
    $MockedDatabaseBasketOrOrder->mock('getUserPassword', sub{
        return 'TestUserPassword';
    });
    $MockedDatabaseBasketOrOrder->mock('getGrandTotalInMinorUnits', sub{
        return '12345';
    });
    $MockedDatabaseBasketOrOrder->mock('getCurrencyID', sub{
        return 'TESTCURR';
    });
    $MockedDatabaseBasketOrOrder->mock('getRedirectUrl', sub{
        return 'redirect.url';
    });
    $MockedDatabaseBasketOrOrder->mock('getHiddenTriggerUrl', sub{
        return 'hiddentrigger.url';
    });

    return $MockedDatabaseBasketOrOrder;
}



sub _createBillsafeArticleList {
    my $BillsafeArticleList = DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList->new();
    my $MockedBillsafeArticleList = Test::MockObject::Extends->new($BillsafeArticleList);

    $MockedBillsafeArticleList->mock('extractArticleList', sub {
        return 'TestArticleList';
    });

    return $MockedBillsafeArticleList;
}



sub _createBillingAddress {
    my $hBillingAddressAttributes = {
        'FullName' => 'TestFullName',
        'Street' => 'TestStreet',
        'Zipcode' => 'TestZipcode',
        'City' => 'TestCity',
        'EMail' => 'TestEMail',
        'Country' => {'Code2' => 'TESTCODE2'},
    };

    my $MockedBillingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('BillingAddress', $hBillingAddressAttributes);
    return $MockedBillingAddress;
}



RunScript( Sub => \&Main );