#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CGI_RequestFactory
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Ipayment::API::CGI::RequestFactory
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CGI_RequestFactory;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use Test::Exception;
use Test::More;
use DE_EPAGES::Ipayment::API::CGI::RequestFactory qw (
    CreateDefaultCapture 
    CreateCaptureWithCustomUserAgentAndResponse
    CreateDefaultCancel
    CreateCancelWithCustomUserAgentAndResponse
);



sub Main {
    Test_CreateDefaultCapture();
    Test_CreateCaptureWithCustomUserAgentAndResponse();
    Test_CreateCaptureWithCustomUserAgentAndResponseNegative();
    Test_CreateDefaultCancel();
    Test_CreateCancelWithCustomUserAgentAndResponse();
    Test_CreateCancelWithCustomUserAgentAndResponseNegative();

    done_testing();
}



sub Test_CreateDefaultCapture {
    my $subTestName = (caller(0))[3];


    my $Request = CreateDefaultCapture();


    lives_ok( sub {TestObject('test', $Request->{'LwpUserAgent'}, 'LWP::UserAgent')},
        "$subTestName - tests if a LWP UserAgent is available");

    lives_ok( sub {TestObject('test', $Request->{'Response'}, 'DE_EPAGES::Ipayment::API::CGI::Response')},
        "$subTestName - tests if a CGI::Response is available");

    lives_ok( sub {TestObject('test', $Request->{'DatabaseBasketOrOrder'}, 'DE_EPAGES::Ipayment::API::Database::BasketOrOrder')},
        "$subTestName - tests if a DatabaseBasketOrOrder is available");
}



sub Test_CreateCaptureWithCustomUserAgentAndResponse {
    my $subTestName = (caller(0))[3];

    my $MockedUserAgent = DE_EPAGES::Test::Mock::EpagesObject->new('LWP::UserAgent', {});
    my $MockedResponse = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::CGI::Response', {});

    my $Request = CreateCaptureWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);


    lives_ok( sub {TestObject('test', $Request->{'LwpUserAgent'}, 'LWP::UserAgent')},
        "$subTestName - tests if a LWP::UserAgent is available");

    lives_ok( sub {TestObject('test', $Request->{'Response'}, 'DE_EPAGES::Ipayment::API::CGI::Response')},
        "$subTestName - tests if a CGI::Response is available");

    lives_ok( sub {TestObject('test', $Request->{'DatabaseBasketOrOrder'}, 'DE_EPAGES::Ipayment::API::Database::BasketOrOrder')},
        "$subTestName - tests if a DatabaseBasketOrOrder is available");
}



sub Test_CreateCaptureWithCustomUserAgentAndResponseNegative {
    my $SubTestName = (caller(0))[3];

    my $MockedUserAgent = '';
    my $MockedResponse  = '';

    throws_ok { CreateCaptureWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse) } (
        qr/Passed Object is not a/,
        "$SubTestName - tests if factory method throws an error if passed objects are of wrong type"
        );
}



sub Test_CreateDefaultCancel {
    my $subTestName = (caller(0))[3];


    my $Request = CreateDefaultCancel();


    lives_ok( sub {TestObject('test', $Request->{'LwpUserAgent'}, 'LWP::UserAgent')},
        "$subTestName - tests if a LWP UserAgent is available");

    lives_ok( sub {TestObject('test', $Request->{'Response'}, 'DE_EPAGES::Ipayment::API::CGI::Response')},
        "$subTestName - tests if a CGI::Response is available");

    lives_ok( sub {TestObject('test', $Request->{'DatabaseBasketOrOrder'}, 'DE_EPAGES::Ipayment::API::Database::BasketOrOrder')},
        "$subTestName - tests if a DatabaseBasketOrOrder is available");
}



sub Test_CreateCancelWithCustomUserAgentAndResponse {
    my $subTestName = (caller(0))[3];

    my $MockedUserAgent = DE_EPAGES::Test::Mock::EpagesObject->new('LWP::UserAgent', {});
    my $MockedResponse = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::CGI::Response', {});

    my $Request = CreateCancelWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);


    lives_ok( sub {TestObject('test', $Request->{'LwpUserAgent'}, 'LWP::UserAgent')},
        "$subTestName - tests if a LWP::UserAgent is available");

    lives_ok( sub {TestObject('test', $Request->{'Response'}, 'DE_EPAGES::Ipayment::API::CGI::Response')},
        "$subTestName - tests if a CGI::Response is available");

    lives_ok( sub {TestObject('test', $Request->{'DatabaseBasketOrOrder'}, 'DE_EPAGES::Ipayment::API::Database::BasketOrOrder')},
        "$subTestName - tests if a DatabaseBasketOrOrder is available");
}



sub Test_CreateCancelWithCustomUserAgentAndResponseNegative {
    my $SubTestName = (caller(0))[3];

    my $MockedUserAgent = '';
    my $MockedResponse  = '';

    throws_ok { CreateCancelWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse) } (
        qr/Passed Object is not a/,
        "$SubTestName - tests if factory method throws an error if passed objects are of wrong type"
        );
}


RunScript( Sub => \&Main );