#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CGI_Response
# state        public
#----------------------------------------------------------------------------------------
# description  Test class use to test the DE_EPAGES::Ipayment::API::CGI::Response
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CGI_Response;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Ipayment::API::CGI::Cancel;
use DE_EPAGES::Ipayment::API::CGI::Response;
use DE_EPAGES::Ipayment::API::CGI::SecurityLayer;
use DE_EPAGES::Ipayment::API::Order::Payment;
use DE_EPAGES::Ipayment::t::unit::MockTools qw ( GetOrder );
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::More;

use strict;

sub test {
    my $self = shift;

    #Testplan
    $self->test_FailCancelTransaction();
    $self->test_SuccessCancelTransaction();

    $self->test_FailCaptureTransaction();
    $self->test_SuccessCaptureTransaction();

    $self->test_CreateTransactionFail();
    $self->test_CreateTransactionSuccessPreauth();
    $self->test_CreateTransactionSuccessAuth();
    $self->test_CreateTransactionSuccessCapture();
    $self->test_CreateTransactionSuccessButUnknownTransStatus();

    $self->test_CreateTransactionSuccessPaypalPaymentStatusCompleted();
    $self->test_CreateTransactionSuccessPaypalPaymentStatusUnknown();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_CreateTransactionSuccessButUnknownTransStatus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'SUCCESS';
    my $Reponse = $self->_createResponse();
    my $LineItemPayment = $Reponse->getLineItemPayment();
    $LineItemPayment->set({'TransStatus' => 'TestStatus'});
    my $ReturnTransactionType = 'TestType';
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$LineItemPayment, $ReturnTransactionType);

    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    ok($WasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after sucessful unknown TransType createtransaction");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after sucessful unknown TransType createtransaction");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after sucessful unknown TransType createtransaction");

    my $ExpectedTransStatus = 'STATUS_FAILED';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus after sucessful unknown TransType createtransaction");

    my $ExpectedPaidOn = undef;
    my $PaidOn = $SecurityLayer->getBasketOrOrder()->get('PaidOn');
    is($PaidOn,$ExpectedPaidOn, "$SubTestName - test if PaidOn is undefined");

    return undef;
}

#----------------------------------------------------------------------------------------
sub test_CreateTransactionSuccessPaypalPaymentStatusCompleted {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'SUCCESS';
    my $Reponse = $self->_createResponse();
    my $LineItemPayment = $Reponse->getLineItemPayment();
    $LineItemPayment->set({'TransStatus' => 'TestStatus'});
    my $ReturnTransactionType = 'auth';
    my $ReturnPaypalPaymentStatus = 'Completed';
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$LineItemPayment, $ReturnTransactionType, $ReturnPaypalPaymentStatus);

    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    ok($WasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after sucessful paypal completed");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after sucessful paypal completed");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after sucessful paypal completed");

    my $ExpectedTransStatus = 'STATUS_PAID';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus is paid after sucessful paypal completed");

    my $ExpectedPaidOn = 'CurrentTime';
    my $PaidOn = $SecurityLayer->getBasketOrOrder()->get('PaidOn');
    is($PaidOn,$ExpectedPaidOn, "$SubTestName - test if PaidOn is set with the current time");

    return undef;
}
#----------------------------------------------------------------------------------------
sub test_CreateTransactionSuccessPaypalPaymentStatusUnknown {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'SUCCESS';
    my $Reponse = $self->_createResponse();
    my $LineItemPayment = $Reponse->getLineItemPayment();
    $LineItemPayment->set({'TransStatus' => 'TestStatus'});
    my $ReturnTransactionType = 'auth';
    my $ReturnPaypalPaymentStatus = 'testStatus';
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$LineItemPayment, $ReturnTransactionType, $ReturnPaypalPaymentStatus);

    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    ok($WasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after sucessful paypal with unknown payment status");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after sucessful paypal with unknown payment status");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after sucessful paypal with unknown payment status");

    my $ExpectedTransStatus = 'STATUS_FAILED';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus after sucessful paypal with unknown payment status");

    my $ExpectedPaidOn = undef;
    my $PaidOn = $SecurityLayer->getBasketOrOrder()->get('PaidOn');
    is($PaidOn,$ExpectedPaidOn, "$SubTestName - test if PaidOn is undef");

    return undef;
}
#----------------------------------------------------------------------------------------
sub test_CreateTransactionSuccessAuth {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'SUCCESS';
    my $Reponse = $self->_createResponse();
    my $LineItemPayment = $Reponse->getLineItemPayment();
    $LineItemPayment->set({'TransStatus' => 'TestStatus'});
    my $ReturnTransactionType = 'auth';
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$LineItemPayment, $ReturnTransactionType);

    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    ok($WasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after sucessful auth createtransaction");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after sucessful auth createtransaction");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after sucessful auth createtransaction");

    my $ExpectedTransStatus = 'STATUS_PAID';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus after sucessful auth createtransaction");

    my $ExpectedPaidOn = 'CurrentTime';
    my $PaidOn = $SecurityLayer->getBasketOrOrder()->get('PaidOn');
    is($PaidOn,$ExpectedPaidOn, "$SubTestName - test if PaidOn is changed to current time");

    return undef;
}

#----------------------------------------------------------------------------------------
sub test_CreateTransactionSuccessCapture {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'SUCCESS';
    my $Reponse = $self->_createResponse();
    my $LineItemPayment = $Reponse->getLineItemPayment();
    $LineItemPayment->set({'TransStatus' => 'TestStatus'});
    my $ReturnTransactionType = 'capture';
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$LineItemPayment, $ReturnTransactionType);

    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    ok($WasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after sucessful capture createtransaction");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after sucessful capture createtransaction");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after sucessful capture createtransaction");

    my $ExpectedTransStatus = 'STATUS_PAID';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus after sucessful capture createtransaction");

    my $ExpectedPaidOn = 'CurrentTime';
    my $PaidOn = $SecurityLayer->getBasketOrOrder()->get('PaidOn');
    is($PaidOn,$ExpectedPaidOn, "$SubTestName - test if PaidOn is changed to current time");

    return undef;
}

#----------------------------------------------------------------------------------------
sub test_CreateTransactionSuccessPreauth {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'SUCCESS';
    my $Reponse = $self->_createResponse();
    my $LineItemPayment = $Reponse->getLineItemPayment();
    $LineItemPayment->set({'TransStatus' => 'TestStatus'});
    my $ReturnTransactionType = 'preauth';
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$LineItemPayment, $ReturnTransactionType);

    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    ok($WasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after sucessful preauth createtransaction");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after sucessful preauth createtransaction");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after sucessful preauth createtransaction");

    my $ExpectedTransStatus = 'STATUS_ACCEPTED';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus after sucessful preauth createtransaction");

    return undef;
}

#----------------------------------------------------------------------------------------
sub test_CreateTransactionFail {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ReturnStatus = 'ReturnStatusFailed';
    my $Reponse = $self->_createResponse();
    my $SecurityLayer = $self->_createMockedSecurityLayer($ReturnStatus,$Reponse->getLineItemPayment());

    my $ExpectedWasSuccessfulTransaction = 0;
    my $WasSuccessfulTransaction = $Reponse->createTransaction($SecurityLayer);
    is($WasSuccessfulTransaction,$ExpectedWasSuccessfulTransaction, "$SubTestName - test if the transaction was successful");

    my $LineItemPayment = $SecurityLayer->getLineItemPayment();

    my $ExpectedTransID = 'MockedTransID';
    my $TransID = $LineItemPayment->get('TransID');
    is($TransID,$ExpectedTransID, "$SubTestName - test TransID after failed createtransaction");

    my $ExpectedTransTimeLastModification = 'CurrentTime';
    my $TransTimeLastModification = $LineItemPayment->get('TransTimeMod');
    is($TransTimeLastModification, $ExpectedTransTimeLastModification, "$SubTestName - test TransTimeMod after failed createtransaction");

    my $ExpectedTransTime = 'CurrentTime';
    my $TransTime = $LineItemPayment->get('TransTime');
    is($TransTime,$ExpectedTransTime, "$SubTestName - test TransTime after failed createtransaction");

    my $ExpectedTransStatus = 'STATUS_FAILED';
    my $TransStatus = $LineItemPayment->get('TransStatus');
    is($TransStatus,$ExpectedTransStatus, "$SubTestName - test TransStatus after failed createtransaction");

    return undef;
}

sub test_SuccessCaptureTransaction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hForm = $self->_createForm('SUCCESS');
    my $Response = $self->_createResponse();

    my $OrderBeforeTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusBeforeTransaction = 'STATUS_ACCEPTED';
    my $TransStatusBeforeTransaction = $OrderBeforeTransaction->get('TransStatus');
    is($TransStatusBeforeTransaction,$ExpectedTransStatusBeforeTransaction, "$SubTestName - test TransStatus before capture transaction");

    $Response->captureTransaction($hForm);

    my $OrderAfterTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusAfterTransaction = 'STATUS_PAID';
    my $TransStatusAfterTransaction = $OrderAfterTransaction->get('TransStatus');
    is($TransStatusAfterTransaction,$ExpectedTransStatusAfterTransaction, "$SubTestName - test TransStatus after capture transaction");

    my $ExpectedTransTimeAfterTransaction = 'CurrentTime';
    my $TransTimeAfterTransaction = $OrderAfterTransaction->get('TransTime');
    is($TransTimeAfterTransaction,$ExpectedTransTimeAfterTransaction, "$SubTestName - test TransTime after capture transaction");

    my $ExpectedPaidOnAfterTransaction = 'CurrentTime';
    my $PaidOnAfterTransaction = $OrderAfterTransaction->container()->parent()->get('PaidOn');
    is($PaidOnAfterTransaction,$ExpectedPaidOnAfterTransaction, "$SubTestName - test PaidOn after capture transaction");

    my $ExpectedCancelledOnAfterTransaction = undef;
    my $CancelledAfterTransaction = $OrderAfterTransaction->container()->parent()->get('CancelledOn');
    is($CancelledAfterTransaction,$ExpectedCancelledOnAfterTransaction, "$SubTestName - test CancelledOn after capture transaction");

    return undef;
}


#----------------------------------------------------------------------------------------
sub test_SuccessCancelTransaction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hForm = $self->_createForm('SUCCESS');
    my $Response = $self->_createResponse();

    my $OrderBeforeTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusBeforeTransaction = 'STATUS_ACCEPTED';
    my $TransStatusBeforeTransaction = $OrderBeforeTransaction->get('TransStatus');
    is($TransStatusBeforeTransaction,$ExpectedTransStatusBeforeTransaction, "$SubTestName - test TransStatus before cancel transaction");

    $Response->cancelTransaction($hForm);

    my $OrderAfterTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusAfterTransaction = 'STATUS_CANCELLED';
    my $TransStatusAfterTransaction = $OrderAfterTransaction->get('TransStatus');
    is($TransStatusAfterTransaction,$ExpectedTransStatusAfterTransaction, "$SubTestName - test TransStatus after cancel transaction");

    my $ExpectedTransTimeAfterTransaction = 'CurrentTime';
    my $TransTimeAfterTransaction = $OrderAfterTransaction->get('TransTime');
    is($TransTimeAfterTransaction,$ExpectedTransTimeAfterTransaction, "$SubTestName - test TransTime after cancel transaction");

    my $ExpectedPaidOnAfterTransaction = undef;
    my $PaidOnAfterTransaction = $OrderAfterTransaction->container()->parent()->get('PaidOn');
    is($PaidOnAfterTransaction,$ExpectedPaidOnAfterTransaction, "$SubTestName - test PaidOn after cancel transaction");

    my $ExpectedCancelledOnAfterTransaction = 'CurrentTime';
    my $CancelledAfterTransaction = $OrderAfterTransaction->container()->parent()->get('CancelledOn');
    is($CancelledAfterTransaction,$ExpectedCancelledOnAfterTransaction, "$SubTestName - test CancelledOn after cancel transaction");


    return undef;
}


#----------------------------------------------------------------------------------------
sub test_FailCaptureTransaction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hForm = $self->_createForm('ERROR');
    my $Response = $self->_createResponse();

    my $OrderBeforeTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusBeforeTransaction = 'STATUS_ACCEPTED';
    my $TransStatusBeforeTransaction = $OrderBeforeTransaction->get('TransStatus');
    is($TransStatusBeforeTransaction,$ExpectedTransStatusBeforeTransaction, "$SubTestName - test TransStatus before capture transaction");

    $Response->captureTransaction($hForm);

    my $OrderAfterTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusAfterTransaction = 'STATUS_FAILED';
    my $TransStatusAfterTransaction = $OrderAfterTransaction->get('TransStatus');
    is($TransStatusAfterTransaction,$ExpectedTransStatusAfterTransaction, "$SubTestName - test TransStatus after capture transaction");

    my $ExpectedTransTimeAfterTransaction = 'CurrentTime';
    my $TransTimeAfterTransaction = $OrderAfterTransaction->get('TransTime');
    is($TransTimeAfterTransaction,$ExpectedTransTimeAfterTransaction, "$SubTestName - test TransTime after capture transaction");

    my $ExpectedPaidOnAfterTransaction = undef;
    my $PaidOnAfterTransaction = $OrderAfterTransaction->container()->parent()->get('PaidOn');
    is($PaidOnAfterTransaction,$ExpectedPaidOnAfterTransaction, "$SubTestName - test PaidOn after capture transaction");

    my $ExpectedCancelledOnAfterTransaction = undef;
    my $CancelledAfterTransaction = $OrderAfterTransaction->container()->parent()->get('CancelledOn');
    is($CancelledAfterTransaction,$ExpectedCancelledOnAfterTransaction, "$SubTestName - test CancelledOn after capture transaction");


    return undef;
}


#----------------------------------------------------------------------------------------
sub test_FailCancelTransaction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hForm = $self->_createForm('ERRRORRR');
    my $Response = $self->_createResponse();

    my $OrderBeforeTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusBeforeTransaction = 'STATUS_ACCEPTED';
    my $TransStatusBeforeTransaction = $OrderBeforeTransaction->get('TransStatus');
    is($TransStatusBeforeTransaction,$ExpectedTransStatusBeforeTransaction, "$SubTestName - test TransStatus before cancel transaction");

    $Response->cancelTransaction($hForm);

    my $OrderAfterTransaction = $Response->getLineItemPayment();

    my $ExpectedTransStatusAfterTransaction = 'STATUS_FAILED';
    my $TransStatusAfterTransaction = $OrderAfterTransaction->get('TransStatus');
    is($TransStatusAfterTransaction,$ExpectedTransStatusAfterTransaction, "$SubTestName - test TransStatus after cancel transaction");

    my $ExpectedTransTimeAfterTransaction = 'CurrentTime';
    my $TransTimeAfterTransaction = $OrderAfterTransaction->get('TransTime');
    is($TransTimeAfterTransaction,$ExpectedTransTimeAfterTransaction, "$SubTestName - test TransTime after cancel transaction");

    my $ExpectedPaidOnAfterTransaction = undef;
    my $PaidOnAfterTransaction = $OrderAfterTransaction->container()->parent()->get('PaidOn');
    is($PaidOnAfterTransaction,$ExpectedPaidOnAfterTransaction, "$SubTestName - test PaidOn after cancel transaction");

    my $ExpectedCancelledOnAfterTransaction = undef;
    my $CancelledAfterTransaction = $OrderAfterTransaction->container()->parent()->get('CancelledOn');
    is($CancelledAfterTransaction,$ExpectedCancelledOnAfterTransaction, "$SubTestName - test CancelledOn after cancel transaction");


    return undef;
}


sub _createForm {
    my $self = shift;
    my ($ReturnStatus) = @_;
    my $hFormParams = {'ret_status' => $ReturnStatus};

    return $hFormParams;
}

sub _createResponse {
    my $self = shift;

    my $Response = DE_EPAGES::Ipayment::API::CGI::Response->new();
    my $MockedResponse = Test::MockObject::Extends->new($Response);

    my $MockedLineItemPaymentWithOrder = GetOrder()->container()->payment();

    $MockedResponse->mock('getLineItemPayment', sub {
        return $MockedLineItemPaymentWithOrder;
    });
    TestExistsMethod($Response,'_LoadObjectByGUID');
    $MockedResponse->mock('_LoadObjectByGUID', sub {
        return $MockedLineItemPaymentWithOrder;
    });

    TestExistsMethod($Response,'_UpdateTransactionStatus');
    $MockedResponse->mock('_UpdateTransactionStatus', sub {
        shift;
        $MockedResponse->{'UpdateTransactionStatus_MockVar'} = \@_;
        my $UpdatedStatus = $MockedResponse->{'UpdateTransactionStatus_MockVar'}[1];
        return $self->_mockedUpdateTransactionStatus($MockedLineItemPaymentWithOrder, $UpdatedStatus);
    });

    return $MockedResponse;
}


sub _mockedUpdateTransactionStatus {
    my $self = shift;
    my ($LineItemPayment, $NewStatus) = @_;

    my $hValues = {'TransStatus' => $NewStatus, 'TransTimeMod' => 'CurrentTime', 'TransTime' => 'CurrentTime'};

    $LineItemPayment->set($hValues);
    return $hValues;

}


sub _createMockedSecurityLayer(){
    my $self = shift;
    my ($ReturnStatus,$MockedLineItemPaymentWithOrder,$ReturnTransactionType, $ReturnPaypalPaymentStatus) = @_;

    my $SecurityLayer = DE_EPAGES::Ipayment::API::CGI::SecurityLayer->new();
    my $MockedSecurityLayer = Test::MockObject::Extends->new($SecurityLayer);

    TestExistsMethod($SecurityLayer,'getReturnStatus');
    $MockedSecurityLayer->mock('getReturnStatus', sub {
        return $ReturnStatus;
    });
    TestExistsMethod($SecurityLayer,'getLineItemPayment');
    $MockedSecurityLayer->mock('getLineItemPayment', sub {
        return $MockedLineItemPaymentWithOrder;
    });
    TestExistsMethod($SecurityLayer,'getTransactionNumber');
    $MockedSecurityLayer->mock('getTransactionNumber', sub {
        return 'MockedTransID';
    });
    TestExistsMethod($SecurityLayer,'getTransactionType');
    $MockedSecurityLayer->mock('getTransactionType', sub {
        return $ReturnTransactionType;
    });
    TestExistsMethod($SecurityLayer,'getBasketOrOrder');
    $MockedSecurityLayer->mock('getBasketOrOrder', sub {
        return $MockedLineItemPaymentWithOrder->container()->parent();
    });
    TestExistsMethod($SecurityLayer,'getPaymentType');
    $MockedSecurityLayer->mock('getPaymentType', sub {
        if(isValid($ReturnPaypalPaymentStatus)) {
            return 'paypal';
        } else {
            return;
        }
    });
    TestExistsMethod($SecurityLayer,'getReturnPaymentStatus');
    $MockedSecurityLayer->mock('getReturnPaymentStatus', sub {
        return $ReturnPaypalPaymentStatus;
    });
}


__PACKAGE__->runScript();
1;
