#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CompiledClient
# state        public
#----------------------------------------------------------------------------------------
# description  Unit Test for DE_EPAGES::Ipayment::API::SOAP::CompiledClient
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CompiledClient;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Ipayment::API::SOAP::CompiledClient;
use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_createClientWithValidInputs();
    $self->test_createClientWithMethodWhichIsNotDefinedInWsdl();
    $self->test_createClientWithInvalidWsdl();

    done_testing();
}



sub test_createClientWithValidInputs {
    my $SubTestName = (caller(0))[3];

    my $ServiceURL = '';
    my $WebserviceMethod = 'testMethod';
    my $CompiledClient = DE_EPAGES::Ipayment::API::SOAP::CompiledClient->new($ServiceURL , $WebserviceMethod);
    $CompiledClient->{'wsdlPath'} = 'CompiledClientTest.wsdl';

    my $WebserviceCall = $CompiledClient->createClient();

    is(ref($WebserviceCall), 'CODE', "$SubTestName - tests if createClient returns a function pointer");
}



sub test_createClientWithMethodWhichIsNotDefinedInWsdl {
    my $SubTestName = (caller(0))[3];

    my $ServiceURL = '';
    my $WebserviceMethod = 'testMethodWhichIsNotDefinedInWsdl';
    my $CompiledClient = DE_EPAGES::Ipayment::API::SOAP::CompiledClient->new($ServiceURL , $WebserviceMethod);
    $CompiledClient->{'wsdlPath'} = 'CompiledClientTest.wsdl';

    throws_ok { $CompiledClient->createClient() } (
        qr/no operation \`testMethodWhichIsNotDefinedInWsdl\' for portType/, ## no critic (ProhibitComplexRegexes)
        "$SubTestName - tests if using webservice with method which is not in wsdl is not possible"
        );
}



sub test_createClientWithInvalidWsdl {
    my $SubTestName = (caller(0))[3];

    my $ServiceURL = '';
    my $WebserviceMethod = '';
    my $CompiledClient = DE_EPAGES::Ipayment::API::SOAP::CompiledClient->new($ServiceURL , $WebserviceMethod);
    $CompiledClient->{'wsdlPath'} = '';

    throws_ok { $CompiledClient->createClient() } (
        qr/known how to interpret XML data/,
        "$SubTestName - tests if using webservice without valid wsdl is not possible"
        );
}

__PACKAGE__->runScript();
1;