#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::Order_Payment
# state        public
#----------------------------------------------------------------------------------------
# description  Test class use to test the DE_EPAGES::Ipayment::API::Order::Payment
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Order_Payment;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Ipayment::API::CGI::Cancel;
use DE_EPAGES::Ipayment::API::Order::Payment;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Ipayment::API::CGI::RequestFactory  qw ( CreateDefaultCapture CreateDefaultCancel );
use DE_EPAGES::Ipayment::API::Constants qw (%CGI_MESSAGES %TRANSLATED_MESSAGES);
use strict;

sub test {
    my $self = shift;

    #Testplan
    $self->test_CancelPayment();
    $self->test_CancelPaymentWithOutCallingCancel();
    $self->test_CancelWith_ErrorNoIpayment();
    $self->test_CancelWith_ErrorUnsupportedAction();
    $self->test_CancelWith_NO_HASH();
    $self->test_CancelWith_NO_REDIRECT_URL();
    $self->test_CancelWithFaillingWithDefaultError();

    $self->test_CapturePayment();
    $self->test_CapturePaymentWithOutCallingCapture();
    $self->test_CaptureWith_ErrorNoIpayment();
    $self->test_CaptureWith_ErrorUnsupportedAction();
    $self->test_CaptureWith_NO_HASH();
    $self->test_CaptureWith_NO_REDIRECT_URL();
    $self->test_CaptureWithFaillingWithDefaultError();

    done_testing();
}
#----------------------------------------------------------------------------------------
# Cancel
#----------------------------------------------------------------------------------------
sub test_CancelPayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $ExpectedStatusIfSuccesfull = $CGI_MESSAGES{'OK'};
    my $Payment = $self->_createOrderPaymentForCancel($ExpectedStatusIfSuccesfull);

    is($Payment->cancel($Order),$ExpectedStatusIfSuccesfull,"$SubTestName - Test if the payment of the order was canceled.");
    is($Payment->getStatus(),$ExpectedStatusIfSuccesfull,"$SubTestName - Test the status of the canceled payment.");
    is($Payment->getMessage(),'',"$SubTestName - Test if is no message to a canceled payment.");
}

#----------------------------------------------------------------------------------------
sub test_CancelPaymentWithOutCallingCancel {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Payment = $self->_createOrderPaymentForCancel();

    is($Payment->getStatus(),undef,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(),undef,"$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CancelWith_ErrorNoIpayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCancelStatus = $CGI_MESSAGES{'NO_IPAYMENT_LINEITEM'};
    my $Payment = $self->_createOrderPaymentForCancel($CIGCancelStatus);

    my $ExpectedStatus = 'error';
    is($Payment->cancel($Order),$ExpectedStatus,"$SubTestName - Test the error status if there is no ipayment lineitem.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'INVALID_ORDER'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CancelWith_ErrorUnsupportedAction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCancelStatus = $CGI_MESSAGES{'UNSUPPORTED_ACTION'};
    my $Payment = $self->_createOrderPaymentForCancel($CIGCancelStatus);

    my $ExpectedStatus = 'error';
    is($Payment->cancel($Order),$ExpectedStatus,"$SubTestName - Test the error status if the action is not supported.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'INVALID_ORDER'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CancelWith_NO_HASH{
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCancelStatus = $CGI_MESSAGES{'NO_HASH'};
    my $Payment = $self->_createOrderPaymentForCancel($CIGCancelStatus);

    my $ExpectedStatus = 'error';
    is($Payment->cancel($Order),$ExpectedStatus,"$SubTestName - Test the error status if the cig webservice dont answer correctly.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'WEBSERVICE_ERROR'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CancelWith_NO_REDIRECT_URL{
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCancelStatus = $CGI_MESSAGES{'NO_REDIRECT_URL'};
    my $Payment = $self->_createOrderPaymentForCancel($CIGCancelStatus);

    my $ExpectedStatus = 'error';
    is($Payment->cancel($Order),$ExpectedStatus,"$SubTestName - Test the error status if the cig webservice response dont contain a redirect URL");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'WEBSERVICE_ERROR'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CancelWithFaillingWithDefaultError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCancelStatus = 'UnknownErrorMessage';
    my $Payment = $self->_createOrderPaymentForCancel($CIGCancelStatus);

    my $ExpectedStatus = 'error';
    is($Payment->cancel($Order),$ExpectedStatus,"$SubTestName - Test the error status if a unknown error appears");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), "$TRANSLATED_MESSAGES{'UNKNOWN_ERROR'}: $CIGCancelStatus","$SubTestName - Test if the MESSAGE of the payment is undef.");
}


#----------------------------------------------------------------------------------------
# capture
#----------------------------------------------------------------------------------------
sub test_CapturePayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $ExpectedStatusIfSuccesfull = $CGI_MESSAGES{'OK'};
    my $Payment = $self->_createOrderPaymentForCapture($ExpectedStatusIfSuccesfull);

    is($Payment->capture($Order),$ExpectedStatusIfSuccesfull,"$SubTestName - Test if the payment of the order was captured.");
    is($Payment->getStatus(),$ExpectedStatusIfSuccesfull,"$SubTestName - Test the status of the captured payment.");
    is($Payment->getMessage(),'',"$SubTestName - Test if is no message to a captured payment.");
}

#----------------------------------------------------------------------------------------
sub test_CapturePaymentWithOutCallingCapture {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Payment = $self->_createOrderPaymentForCapture();

    is($Payment->getStatus(), undef, "$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), undef, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CaptureWith_ErrorNoIpayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCaptureStatus = $CGI_MESSAGES{'NO_IPAYMENT_LINEITEM'};
    my $Payment = $self->_createOrderPaymentForCapture($CIGCaptureStatus);

    my $ExpectedStatus = 'error';
    is($Payment->capture($Order), $ExpectedStatus, "$SubTestName - Test the error status if there is no ipayment lineitem.");
    is($Payment->getStatus(), $ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'INVALID_ORDER'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CaptureWith_ErrorUnsupportedAction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCaptureStatus = $CGI_MESSAGES{'UNSUPPORTED_ACTION'};
    my $Payment = $self->_createOrderPaymentForCapture($CIGCaptureStatus);

    my $ExpectedStatus = 'error';
    is($Payment->capture($Order),$ExpectedStatus,"$SubTestName - Test the error status if the action is not supported.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'INVALID_ORDER'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}

#----------------------------------------------------------------------------------------
sub test_CaptureWith_NO_HASH {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCaptureStatus = $CGI_MESSAGES{'NO_HASH'};
    my $Payment = $self->_createOrderPaymentForCapture($CIGCaptureStatus);

    my $ExpectedStatus = 'error';
    is($Payment->capture($Order),$ExpectedStatus,"$SubTestName - Test the error status if the cig webservice dont answer correctly.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'WEBSERVICE_ERROR'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}
#----------------------------------------------------------------------------------------
sub test_CaptureWith_NO_REDIRECT_URL {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCaptureStatus = $CGI_MESSAGES{'NO_REDIRECT_URL'};
    my $Payment = $self->_createOrderPaymentForCapture($CIGCaptureStatus);

    my $ExpectedStatus = 'error';
    is($Payment->capture($Order),$ExpectedStatus,"$SubTestName - Test the error status if the cig webservice response dont contain a redirect URL.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), $TRANSLATED_MESSAGES{'WEBSERVICE_ERROR'}, "$SubTestName - Test if the MESSAGE of the payment is undef.");
}
#----------------------------------------------------------------------------------------
sub test_CaptureWithFaillingWithDefaultError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createOrder();
    my $CIGCaptureStatus = 'UnknownErrorMessage';
    my $Payment = $self->_createOrderPaymentForCapture($CIGCaptureStatus);

    my $ExpectedStatus = 'error';
    is($Payment->capture($Order),$ExpectedStatus,"$SubTestName - Test the error status if a unknown error appears.");
    is($Payment->getStatus(),$ExpectedStatus,"$SubTestName - Test if the STATUS of the payment is undef.");
    is($Payment->getMessage(), "$TRANSLATED_MESSAGES{'UNKNOWN_ERROR'}: $CIGCaptureStatus", "$SubTestName - Test if the MESSAGE of the payment is undef.");
}



#----------------------------------------------------------------------------------------
# test helper
#----------------------------------------------------------------------------------------
sub _createOrderPaymentForCancel {
    my $self = shift;
    my ($ExpectedStatus) = @_;

    my $CgiCancel = $self->_createCgiCancel($ExpectedStatus);
    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->setCancelObject($CgiCancel);

    return $Payment;
}

#----------------------------------------------------------------------------------------
sub _createOrderPaymentForCapture {
    my $self = shift;
    my ($ExpectedStatus) = @_;

    my $CgiCapture = $self->_createCgiCapture($ExpectedStatus);
    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->setCaptureObject($CgiCapture);

    return $Payment;
}

#----------------------------------------------------------------------------------------
sub _createCgiCancel {
    my $self = shift;
    my ($ReturnValue) = @_;

    my $MethodName = 'cancelMoney';
    my $CgiCancel = CreateDefaultCancel();
    TestExistsMethod($CgiCancel,$MethodName);
    my $MockedCgiCancel = Test::MockObject::Extends->new($CgiCancel);
    $MockedCgiCancel->mock($MethodName, sub { return $ReturnValue; });

    return $MockedCgiCancel;
}

#----------------------------------------------------------------------------------------
sub _createCgiCapture {
    my $self = shift;
    my ($ReturnValue) = @_;

    my $MethodName = 'captureMoney';
    my $CgiCapture = CreateDefaultCapture();
    TestExistsMethod($CgiCapture,$MethodName);
    my $MockedCgiCancel = Test::MockObject::Extends->new($CgiCapture);
    $MockedCgiCancel->mock($MethodName, sub { return $ReturnValue; });

    return $MockedCgiCancel;
}

#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;

    my $OrderObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder',{});

    return $OrderObject;
}

__PACKAGE__->runScript();
1;
