#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::Response_Parser
# state        public
#----------------------------------------------------------------------------------------
# description  Unit Test for Response_Parser.pm
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Response_Parser;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::SOAP::CompiledClient;
use DE_EPAGES::Ipayment::API::SOAP::Response::SessionIdParser;
use DE_EPAGES::Ipayment::API::SOAP::WebServiceClient;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Headers;
use HTTP::Response;
use LWP::UserAgent;
use Test::MockObject::Extends;
use Test::More;
use XML::Compile::WSDL11;
use DE_EPAGES::Ipayment::t::unit::TestTools qw (
    ImportTestXmlFile
    CreatePositiveXMLHttpResponse
    CreateMockedUserAgent
    CreateCompiledClientWithCustomUserAgent
);

use strict;

use constant TEST_RESPONSE_PATH => "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Ipayment/t/TestResponses/";



sub Main {
    test_GetSessionIdWithoutCallingParse();
    test_ParsePositiveSoapResponse();
    test_ParseNegativeSoapResponse();

    done_testing();
}



sub test_GetSessionIdWithoutCallingParse {
    my $SubTestName = (caller(0))[3];

    my $ExpectedSessionID = undef;
    my $SessionIdParser = DE_EPAGES::Ipayment::API::SOAP::Response::SessionIdParser->new();

    my $SessionId = $SessionIdParser->getSessionId();
    is($SessionId, $ExpectedSessionID, "$SubTestName - get undef instead of parsed sessionId - undef");
}



sub test_ParsePositiveSoapResponse {
    my $SubTestName = (caller(0))[3];

    my $TestSoapResponse = _GetPositiveTestSoapResponseFromXml();
    my $SessionIdParser = DE_EPAGES::Ipayment::API::SOAP::Response::SessionIdParser->new();

    my $ParseStatus = $SessionIdParser->parse($TestSoapResponse);
    ok($ParseStatus, "$SubTestName - check if response contains a sessionId");

    my $SessionId = $SessionIdParser->getSessionId();
    my $ExpectedSessionID = '0a78161fj16crEtFnTH1IGdHAKWI5nAI';
    is($SessionId, $ExpectedSessionID, "$SubTestName - get parsed sessionId - string with SessionId");
}



sub test_ParseNegativeSoapResponse {
    my $SubTestName = (caller(0))[3];

    my $TestSoapResponse = _GetNegativeTestSoapResponseFromXml();
    my $SessionIdParser = DE_EPAGES::Ipayment::API::SOAP::Response::SessionIdParser->new();

    my $ParseStatus = $SessionIdParser->parse($TestSoapResponse);
    is($ParseStatus, 0, "$SubTestName - check if response contains no sessionId");

    my $SessionId = $SessionIdParser->getSessionId();
    my $ExpectedSessionID = undef;
    is($SessionId, $ExpectedSessionID, "$SubTestName - get undef instead of parsed sessionId - undef");
}



sub _GetPositiveTestSoapResponseFromXml {
    my $TestSoapResponse = ImportTestXmlFile(TEST_RESPONSE_PATH.'CreateSessionPositiveResponse.xml');


    return _GetSoapResponseHashFromSoapXmlResponse($TestSoapResponse);
}



sub _GetNegativeTestSoapResponseFromXml {
    my $TestSoapResponse = ImportTestXmlFile(TEST_RESPONSE_PATH.'CreateSessionNegativeResponse.xml');
    my $SoapResponseHashFromSoapXml = _GetSoapResponseHashFromSoapXmlResponse($TestSoapResponse);
    return $SoapResponseHashFromSoapXml;
}



sub _GetSoapResponseHashFromSoapXmlResponse {
    my ($XmlResponse) = @_;

    my $MockedCompiledClient = _CreateMockedCompiledClientWithCustomSoapResponse( $XmlResponse );
    my $WebServiceClient = DE_EPAGES::Ipayment::API::SOAP::WebServiceClient->new($MockedCompiledClient);
    my $MockedWebserviceClient = Test::MockObject::Extends->new($WebServiceClient);
    $MockedWebserviceClient->mock('_LogPayment', sub {
        return undef;
    });

    my $MockedHashBuilder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder', {});
    $MockedHashBuilder->mock('getRequestData', sub {
        return {};
    });

    my $MockedValidator = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::SOAP::Request::Validator', {});
    $MockedValidator->mock('isValidHash', sub {
        return 1;
    });
    $MockedWebserviceClient->setValidator($MockedValidator);


    # No error handling needed, because we are just interested in the test results
    ## no critic (RequireExistsError)
    my $hPositiveTestSoapResponse = {};
    eval {
        $hPositiveTestSoapResponse = $MockedWebserviceClient->doRequest($MockedHashBuilder);
    };
    ## use critic (RequireExistsError)

    return $hPositiveTestSoapResponse;
}



sub _CreateMockedCompiledClientWithCustomSoapResponse {
    my ($XmlResponse) = @_;

    my $WebserviceMethod = 'createSession';
    my $ServiceURL = 'https://ipayment.de/service/3.0/?wsdl';

    my $Response = CreatePositiveXMLHttpResponse($XmlResponse);
    my $UserAgent = CreateMockedUserAgent($Response);
    $UserAgent->agent( 'ePages ' );
    my $MockedCompiledClient = CreateCompiledClientWithCustomUserAgent($ServiceURL, $WebserviceMethod, $UserAgent);

    return $MockedCompiledClient;
}


RunScript( Sub => \&Main );