#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::Session;
# state        public
#----------------------------------------------------------------------------------------
# description  Unit tests session.pm
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Session;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::Session;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;

use Test::More;

sub TestPlan {
    test_BasketOrOrderIsNotComplete();
    test_createIpaymentTestSessionAndStoreIdInDatabase();
    test_createIpaymentLiveSessionAndStoreIdInDatabase();

    done_testing();
}

sub test_BasketOrOrderIsNotComplete {
    my $SubTestName = (caller(0))[3];
    my $DatabaseBasketOrOrder = Test::MockObject->new( );
        $DatabaseBasketOrOrder->mock('extractFromBasketOrOrder',sub {
        return 0;
    });
    my $IsValidRequestData = 0;
    my $Session = _GetSessionWithOutBindings($DatabaseBasketOrOrder,$IsValidRequestData);

    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase();

    is($Status, 0, "$SubTestName - If the BasketOrOrder is not complete we get a undef and leave");
}

sub test_createIpaymentTestSessionAndStoreIdInDatabase {
    my $SubTestName = (caller(0))[3];

    my $LineItemPayment = _GetLineItemPayment();
    my $IsTestTransaction = 1;
    my $DatabaseBasketOrOrder = _GetDatabaseBasketOrOrder($LineItemPayment, $IsTestTransaction);
    my $IsValidRequestData = 1;
    my $Session = _GetSessionWithOutBindings($DatabaseBasketOrOrder,$IsValidRequestData);

    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase();

    is($Status, 1, "$SubTestName - checks if the operation works fine");
    is($LineItemPayment->get('SessionID'),123456789,"$SubTestName - tests if we get the correct SessionID") ; ## no critic (ProhibitMagicNumbers)
    is($Session->{'WebServiceUrl_MockVar'},'mockTESTURL',"$SubTestName - tests if the code Took the right Endpoint URL" );
}

sub test_createIpaymentLiveSessionAndStoreIdInDatabase {
    my $SubTestName = (caller(0))[3];

    my $LineItemPayment = _GetLineItemPayment();
    my $IsTestTransaction = 0;
    my $DatabaseBasketOrOrder = _GetDatabaseBasketOrOrder($LineItemPayment, $IsTestTransaction);
    my $IsValidRequestData = 1;
    my $Session = _GetSessionWithOutBindings($DatabaseBasketOrOrder,$IsValidRequestData);

    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase();

    is($Status, 1, "$SubTestName - checks if the operation works fine");
    is($LineItemPayment->get('SessionID'),123456789,"$SubTestName - tests if we get the correct SessionID") ;## no critic (ProhibitMagicNumbers)
    is($Session->{'WebServiceUrl_MockVar'},'mockURL',"$SubTestName - tests if the code Took the right Endpoint URL" );
}

# test helpers
#----------------------------------------------------------------------------------------
sub _GetSessionWithOutBindings{
    my ($DatabaseBasketOrOrder, $IsValidRequestData ) = @_;
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    $Session = Test::MockObject::Extends->new( $Session );
    $Session->mock('_createDatabaseBasketOrOrder',sub {
        return $DatabaseBasketOrOrder;
    });
    $Session->mock('_createHashbuilder',sub {
        return _GetHashbuilder($IsValidRequestData);
    });
    $Session->mock('_createCompiledClient',sub {
        my $self = shift;
        my ($WebServiceUrl) = @_;
        $self->{'WebServiceUrl_MockVar'} = $WebServiceUrl;
        return undef;
    });
    $Session->mock('_createWebServiceClient',sub {
        return _GetWebServiceClient();
    });
    $Session->mock('_createSessionIdParser',sub {
        return _GetSessionIDParser();
    });
    $Session->mock('_GetWebServiceUrl',sub {
        return _GetWebServiceUrl($DatabaseBasketOrOrder->getIsTestTransaction());
    });
}
#----------------------------------------------------------------------------------------
sub _GetWebServiceUrl ($) {
    my ($IsInTestMode) = @_;
    if ( $IsInTestMode) {
        return 'mockTESTURL';
    } else {
        return 'mockURL';
    }
}
#----------------------------------------------------------------------------------------
sub _GetHashbuilder {
    my ($IsValidRequestData) = @_;
    my $SessionParser = Test::MockObject->new( );
    $SessionParser->mock('generateRequestData',sub {
        return $IsValidRequestData;
    });
}
#----------------------------------------------------------------------------------------
sub _GetSessionIDParser {
    my $SessionParser = Test::MockObject->new( );
    $SessionParser->mock('parse',sub {
        return 1;
    });
    $SessionParser->mock('getSessionId',sub {
        return 123456789;## no critic (ProhibitMagicNumbers)
    });
}
#----------------------------------------------------------------------------------------
sub _GetDatabaseBasketOrOrder{
    my ($LineItemPayment , $IsTestTransaction) = @_;
    my $DatabaseBasketOrOrder = Test::MockObject->new( );
    $DatabaseBasketOrOrder->mock('extractFromBasketOrOrder',sub {
        return 1;
    });
    $DatabaseBasketOrOrder->mock('getIsTestTransaction',sub {
        return $IsTestTransaction;
    });
    $DatabaseBasketOrOrder->mock('getLineItemPaymentIpayment',sub {
        return $LineItemPayment;
    });
}
#----------------------------------------------------------------------------------------
sub _GetLineItemPayment {
    my ($Arguments) = @_;
         my $hAttributes = {};
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
}
#----------------------------------------------------------------------------------------
sub _GetWebServiceClient {
    my $WebServiceClient = Test::MockObject->new( );
    $WebServiceClient->mock('doRequest',sub {
        return '';
    });
    return $WebServiceClient;
}


RunScript( Sub => \&TestPlan );