#========================================================================================
# package      DE_EPAGES::Test::t::Mock_EpagesObject
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Test class for Mock::EpagesObject.pm
#========================================================================================
package DE_EPAGES::Test::t::Mock_EpagesObject;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_DefinedValues();
    $self->test_UndefinedValues();
    $self->test_UndefinedChildValues();

    done_testing();
}

sub test_DefinedValues {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = {
        'Alias' => 'MyAlias',
        'ID' => 'MyID',
        'GUID' => 'MyGUID',
        'Parent' => 'MyParent',
        'instanceOf' => 'MyInstanceOf',
        'Site' => 'MySite',
        'LineItemContainer' => 'MyLineItemContainer',
        'DummyGetWithParent' => 'MyDummyGetWithParent',
        'Children' => 'MyChildren',
        'PathString' => 'MyPathString',
        'Child' => {'FirstChildName' => 'FirstChildValue', 'SecondChildName' => 'SecondChildValue'}
    };
    my $PackageName = 'DE_EPAGES::Test::Mock::EpagesObject';
    my $EpagesObject = DE_EPAGES::Test::Mock::EpagesObject->new($PackageName, $hAttributes);

    lives_ok(sub { TestObject('EpagesObject',$EpagesObject, 'DE_EPAGES::Test::Mock::EpagesObject'); },
        "$SubTestName - Test if the object is EpagesObject.");
    ok($EpagesObject->isa($PackageName),"$SubTestName - Test if the MOCKED Object is instance of the Class.");
    ok($EpagesObject->instanceOf('MyInstanceOf'),"$SubTestName - Test if the Object is instance of the Class.");

    is($EpagesObject->alias(),'MyAlias',"$SubTestName - Test if the Alias was defined.");
    is($EpagesObject->id(),'MyID',"$SubTestName - Test if the ID was defined.");
    is($EpagesObject->guid(),'MyGUID',"$SubTestName - Test if the GUID was defined.");
    is($EpagesObject->parent(),'MyParent',"$SubTestName - Test if the Parent was defined.");
    is($EpagesObject->getSite(), 'MySite',"$SubTestName - Test if the Site was defined.");
    is($EpagesObject->container(), 'MyLineItemContainer',"$SubTestName - Test if the LineItemContainer was defined.");
    is($EpagesObject->children(), 'MyChildren',"$SubTestName - Test if the Children was defined.");
    is($EpagesObject->pathString(), 'MyPathString',"$SubTestName - Test if the PathString was defined.");

    is($EpagesObject->getWithParent('DummyGetWithParent'), 'MyDummyGetWithParent',"$SubTestName - Test if the GetWithParent was defined.");
    is($EpagesObject->child('FirstChildName'), 'FirstChildValue',"$SubTestName - Test if the child get the expected value by name 'FirstChildName'");
    is($EpagesObject->child('SecondChildName'), 'SecondChildValue',"$SubTestName - Test if the child get the expected value by name 'SecondChildName'");
}


sub test_UndefinedValues {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = {};
    my $PackageName = 'DE_EPAGES::Test::Mock::EpagesObject';
    my $EpagesObject = DE_EPAGES::Test::Mock::EpagesObject->new($PackageName, $hAttributes);

    throws_ok{ TestObject('EpagesObject',$EpagesObject, 'NotEpagesObject') }
        ( qr/WRONG_CLASS/, "$SubTestName - Test if the object is NOT an instance of NotEpagesObject." );
    throws_ok { $EpagesObject->instanceOf('MyInstanceOf') } ( qr/Attribute instanceOf doesn/, "$SubTestName - Test if the instanceOf was undefined." );

    throws_ok { $EpagesObject->alias() } ( qr/Attribute Alias doesn/, "$SubTestName - Test if the Alias was undefined." );
    throws_ok { $EpagesObject->id() } ( qr/Attribute ID doesn/, "$SubTestName - Test if the ID was undefined." );
    throws_ok { $EpagesObject->guid() } ( qr/Attribute GUID doesn/, "$SubTestName - Test if the GUID was undefined." );
    throws_ok { $EpagesObject->parent() } ( qr/Attribute Parent doesn/, "$SubTestName - Test if the Parent was undefined." );
    throws_ok { $EpagesObject->getSite() } ( qr/Attribute Site doesn/, "$SubTestName - Test if the Site was undefined." );
    throws_ok { $EpagesObject->container() } ( qr/Attribute LineItemContainer doesn/, "$SubTestName - Test if the LineItemContainer was undefined." );
    throws_ok { $EpagesObject->children() } ( qr/Attribute Children doesn/, "$SubTestName - Test if the Children was undefined." );
    throws_ok { $EpagesObject->pathString() } ( qr/Attribute PathString doesn/, "$SubTestName - Test if PathString was undefined." );

    throws_ok { $EpagesObject->getWithParent('DummyGetWithParent') } ( qr/Attribute DummyGetWithParent doesn/, "$SubTestName - Test if the DummyGetWithParent was undefined." );
    throws_ok { $EpagesObject->child('UndefChild') } ( qr/Attribute Child doesn/, "$SubTestName - Test if the child was undefined, because was not set on the EpagesObject creation." );
}

sub test_UndefinedChildValues {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = {
        'Child' => {'ChildName' => 'ChildValue'}
    };
    my $ClassName = 'EpagesObject';
    my $EpagesObject = DE_EPAGES::Test::Mock::EpagesObject->new($ClassName, $hAttributes);

    throws_ok { $EpagesObject->child('UndefChild') } ( qr/NoResults_ChildAlias/, "$SubTestName - Test if the object does not contain the expected child." );
    throws_ok { $EpagesObject->child() } ( qr/PARAMETER_MISSING: Missing parameter Alias/, "$SubTestName - Test if the object does not load child without name." );
}

__PACKAGE__->runScript();
1;